@@ -53,6 +53,7 @@ class Drone(object):
         extract_keys(work_dir, config)
         self.work_dir = work_dir
         self.config = config
+        self.config_file = os.path.join(work_dir, 'beeswarmcfg.json')
         self.key = key
         self.cert = cert
         self.id = self.config['general']['id']
@@ -81,6 +82,7 @@ class Drone(object):
                 logger.warning('Could not fetch public ip: {0}'.format(e))
         else:
             self.ip = ''
+        self.greenlets = []
 
     def start(self):
         """ Starts services. """
@@ -96,8 +98,10 @@ class Drone(object):
 
         self.outgoing_msg_greenlet = gevent.spawn(self.outgoing_server_comms, server_public,
                                                   client_public, client_secret)
+        self.outgoing_msg_greenlet.link_exception(self.on_exception)
         self.incoming_msg_greenlet = gevent.spawn(self.incoming_server_comms, server_public,
                                                   client_public, client_secret)
+        self.incoming_msg_greenlet.link_exception(self.on_exception)
 
         self.config_url_dropper_greenlet = gevent.spawn(self.config_url_drop_poller)
 
@@ -109,7 +113,7 @@ class Drone(object):
         Restarts the drone
         """
 
-        with open('beeswarmcfg.json', 'r') as config_file:
+        with open(self.config_file, 'r') as config_file:
             self.config = json.load(config_file, object_hook=asciify)
 
         mode = None
@@ -123,19 +127,27 @@ class Drone(object):
         if mode:
             self.drone = mode(self.work_dir, self.config)
             self.drone_greenlet = gevent.spawn(self.drone.start)
+            self.drone_greenlet.link_exception(self.on_exception)
             logger.info('Drone configured and running'.format(self.id))
 
     def stop(self):
         """Stops services"""
         logging.debug('Stopping drone, hang on.')
         if self.drone is not None:
+            self.drone_greenlet.unlink(self.on_exception)
             self.drone.stop()
+            self.drone_greenlet.kill()
             self.drone = None
         # just some time for the drone to powerdown to be nice.
         gevent.sleep(2)
         if self.drone_greenlet is not None:
             self.drone_greenlet.kill(timeout=5)
 
+    def on_exception(self, dead_greenlet):
+        logger.error('Stopping because {0} died: {1}'.format(dead_greenlet, dead_greenlet.exception))
+        self.stop()
+        sys.exit(1)
+
     # command from server
     def incoming_server_comms(self, server_public, client_public, client_secret):
         context = beeswarm.shared.zmq_context
@@ -180,7 +192,7 @@ class Drone(object):
                     send_zmq_push(SocketNames.SERVER_RELAY, '{0}'.format(Messages.PING))
                     self.config_received.set()
                     config = json.loads(data)
-                    with open('beeswarmcfg.json', 'w') as local_config:
+                    with open(self.config_file, 'w') as local_config:
                         local_config.write(json.dumps(config, indent=4))
                     self.stop()
                     self._start_drone()
@@ -274,7 +286,7 @@ class Drone(object):
                     config_url = open(_file).read()
                 logger.info('Found dropped api config url in {0}, with content: {1}.'.format(self.work_dir, config_url))
                 os.remove(dropped_config_url_file)
-                extract_config_from_api(config_url)
+                extract_config_from_api(config_url, self.config_file)
                 self.stop()
                 self._start_drone()
 
